import controlP5.*;
import peasy.*;
import processing.opengl.*;

PeasyCam cam;
ControlP5 cp5;
ListBox l;

// Data parameters
Table dataTable;
int rowNums, colNums;
int[] MaxCheckoutCount, MinCheckoutCount;
String[] sevenDeadlySins; 

// Drawing setting
float overallRadius, panelDistance;
PVector[] Points; 
int presentYear;
float interpolatorYear;
Integrator[] interpolator;
float rotateX, rotateY, rotateZ;

// Color & fonts
color[] colors;
PFont[] fonts;

// Control parameters
int viewMode, whichSinToSee;
boolean showOtherYear, showBook, showVideo, showAudio, rotateFree, autoRotate, showGUI, onlyStillFrame;
boolean[] titleSelected;




void setup() {
  fullScreen(P3D);
  //size(800, 800, P3D);
  cam = new PeasyCam(this, 250);
  cam.setWheelScale(0.1);
  cam.setDistance(1600);


  //Initialization for data
  //
  dataTable = loadTable("query_result.csv", "header");  
  rowNums = dataTable.getRowCount();
  colNums = dataTable.getColumnCount();

  MaxCheckoutCount = new int[7];
  MinCheckoutCount = new int[7];
  for (int i = 0; i< 7; i++) {
    MaxCheckoutCount[i] = 0;
    MinCheckoutCount[i] = 100000;
  }
  for (int i = 0; i<rowNums; i++) {
    TableRow row = dataTable.getRow(i);
    int index = whichDeadlySin(row.getString(2)); 
    int checkoutCount = row.getInt(4);
    if (checkoutCount > MaxCheckoutCount[index]) MaxCheckoutCount[index] = checkoutCount;
    if (checkoutCount < MinCheckoutCount[index]) MinCheckoutCount[index] = checkoutCount;
  }

  sevenDeadlySins = new String[7];
  sevenDeadlySins[0] = "Lust";
  sevenDeadlySins[1] = "Gluttony";
  sevenDeadlySins[2] = "Greed";
  sevenDeadlySins[3] = "Wrath";
  sevenDeadlySins[4] = "Sloth";
  sevenDeadlySins[5] = "Envy";
  sevenDeadlySins[6] = "Pride";


  // Initializaiton for drawing 
  //
  rotateX = 0;
  rotateY = 0;
  rotateZ = 0;

  viewMode = 0;
  showOtherYear = true;
  showBook = true;
  showVideo = true;
  showAudio = true;
  rotateFree = true;
  showGUI = true;
  autoRotate = false;
  onlyStillFrame = false;

  titleSelected = new boolean[rowNums];
  for (int i = 0; i<rowNums; i++) {
    titleSelected[i] = false;
  }

  overallRadius = 400;
  panelDistance = 60;
  presentYear = 1990;
  interpolator = new Integrator[4];
  interpolator[0] = new Integrator(presentYear, 0.4, 0.08);
  interpolator[1] = new Integrator(rotateX, 0.4, 0.16);
  interpolator[2] = new Integrator(rotateY, 0.4, 0.16);
  interpolator[3] = new Integrator(rotateZ, 0.4, 0.16);

  Points = new PVector[7];
  for (int i = 0; i<7; i++) {
    float x, y;
    x = overallRadius*cos(i*2*PI/7);
    y = overallRadius*sin(i*2*PI/7);
    Points[i] = new PVector(x, y, 0);
  }

  whichSinToSee = 0;

  // Initialize colors & fonts
  //
  colors = new color[10];
  colors[0] = #0A86D0;
  colors[1] = #134ED5;
  colors[2] = #6D5CAF;
  colors[3] = #F2000D;
  colors[4] = #EB6002;
  colors[5] = #FFBC00;
  colors[6] = #019C69;
  colors[7] = #D42020;
  colors[8] = #FFFFFF;
  colors[9] = #000000;

  fonts = new PFont[3];
  fonts[0] = loadFont("OratorStd-48.vlw");
  fonts[1] = loadFont("OratorStd-12.vlw");

  controllerSetup();

  noLights();
  smooth();
}

void draw() {

  background(255);

  for (int i = 0; i < interpolator.length; i++) {
    interpolator[i].update();
  }
  interpolatorYear = interpolator[0].value;

  if (!rotateFree)
    cam.setRotations(interpolator[1].value, interpolator[2].value, interpolator[3].value);
  else
    setRotationInterpolator(cam.getRotations()[0], cam.getRotations()[1], cam.getRotations()[2]);

  for (int i = 0; i<rowNums; i++) {
    if ((l.getItem(i).get("state").toString().equals("true"))) {  
      titleSelected[i] = true;
    } else
      titleSelected[i] = false;
  }

  drawStillFrame();
  if (!onlyStillFrame) {
    for (int i= 0; i<rowNums; i++) {    
      drawTableRow(i);
    }
  }

  if (autoRotate) {
    if (viewMode == 2)cam.rotateX(0.002);
    if (viewMode == 3)cam.rotateY(0.002);
  }
  if (showGUI) gui();
}


void drawTableRow(int i) {
  TableRow row = dataTable.getRow(i);
  String type = row.getString(1);
  String theSin = row.getString(2);
  int year = row.getInt(3);
  if (year<1990) year = 1990;
  int index = whichDeadlySin(theSin);

  // Judgement about itemType
  // If selected, then show.
  //
  boolean shouldShow = false;
  if (type.equals("Book") && showBook) shouldShow = true;
  if (type.equals("Video") && showVideo) shouldShow = true;
  if (type.equals("Audio") && showAudio) shouldShow = true;

  if (shouldShow) {
    float percent = sqrt(sqrt(map(row.getFloat(4), MinCheckoutCount[index], MaxCheckoutCount[index], 0, 1)));
    float percent1 = map(percent, 0, 1, 0.1, 0.9);
    PVector p = PVector.lerp(Points[index], Points[(index+1)%7], percent1);
    float z = (year - interpolatorYear) * panelDistance;

    beginShape(LINES);
    colorMode(HSB, 255);
    color theColor = color(hue(colors[index]), saturation(colors[index])*(percent*0.3+0.7), brightness(colors[index]));

    // Judgement for each row's diplay mode
    //
    int mode = 0;
    if (whichSinToSee == (index+1) || whichSinToSee == 0) 
      mode = 2;
    else 
    mode = 4;
    if (titleSelected[i]==true) mode = 3;
    else {
      if ( mode != 4) {
        if (presentYear == year) mode = 1;
        else {
          if (showOtherYear ) mode = 2;
          else mode =4;
        }
      }
    }

    // Display according to its diplay mode
    //
    switch (mode) {

    case 1:
      // #1 YearSelected
      stroke(theColor, 100);
      strokeWeight(1);
      vertex(p.x, p.y, p.z);
      stroke(theColor, 250);
      strokeWeight(max(10*percent, 1));
      vertex(0, 0, z);
      break;

    case 2:
      // #2 Visible
      stroke(theColor, 100*(percent*0.7+0.3));
      strokeWeight(1);
      vertex(p.x, p.y, p.z);
      stroke(theColor, 250*(percent*0.7+0.3));
      strokeWeight(max(10*percent, 1));
      vertex(0, 0, z);
      break;

    case 3:
      // #3 TitleSelected
      stroke(colors[9], 100);
      strokeWeight(3);
      vertex(p.x, p.y, p.z);
      stroke(colors[9], 250);
      strokeWeight(max(10*percent, 3));
      vertex(0, 0, z);
      break;

    case 4 :
      // #4 Invisible
      stroke(hue(colors[index]), saturation(colors[index])*0.1, brightness(colors[index])*0.1, 15);
      strokeWeight(0.5);
      vertex(p.x, p.y, p.z);
      strokeWeight(0.5);
      vertex(0, 0, z);
    }
    endShape();


    // Show titles of items published at selected year
    //
    if ((year == presentYear && mode != 4 )|| titleSelected[i]) { 
      pushMatrix();
      translate(p.x, p.y, p.z);
      rotateY(PI/2);
      rotateX(((7-index)+0.5)*2*PI/7 - PI/4);
      rotateZ(PI);
      fill(colors[9], 200);
      textFont(fonts[0], 8);
      textAlign(LEFT);
      String title = row.getString(0);
      text(str(row.getInt(4))+"  "+title, 0, 0, 0);
      popMatrix();
    }
  }
}

// A mapping between index and sins
//
int whichDeadlySin(String theSin) {
  int index=0;
  if (theSin.equals("Lust"))     index = 0;
  if (theSin.equals("Gluttony")) index = 1;
  if (theSin.equals("Greed"))    index = 2;
  if (theSin.equals("Wrath"))    index = 3; 
  if (theSin.equals("Sloth"))    index = 4;
  if (theSin.equals("Envy"))     index = 5;
  if (theSin.equals("Pride"))    index = 6;
  return index;
}